@@ -690,7 +690,13 @@ class Adapter(object):
                 state = state.read()
             if state:
                 from_count += 1
-                to_oids = get_references(str(state))
+                try:
+                    to_oids = get_references(str(state))
+                except:
+                    log.error("pre_pack: can't unpickle "
+                        "object %d in transaction %d; state length = %d" % (
+                        from_oid, tid, len(state)))
+                    raise
                 for to_oid in to_oids:
                     add_rows.append((from_oid, tid, to_oid))
 
@@ -790,6 +796,10 @@ class Adapter(object):
                         self._release_commit_lock(cursor)
                         # Add a delay based on the configured duty cycle.
                         elapsed = time.time() - start
+                        if elapsed == 0.0:
+                            # Compensate for low timer resolution by
+                            # assuming that at least 10 ms elapsed.
+                            elapsed = 0.01
                         duty_cycle = options.pack_duty_cycle
                         if duty_cycle > 0.0 and duty_cycle < 1.0:
                             delay = min(options.pack_max_delay,
