@@ -21,6 +21,7 @@ from relstorage.tests import fakecache
 from ZODB.DB import DB
 from ZODB.utils import p64
 from ZODB.FileStorage import FileStorage
+from persistent import Persistent
 from persistent.mapping import PersistentMapping
 import transaction
 
@@ -234,13 +235,12 @@ class RelStorageTests(
         # Store an object, cache it, then retrieve it from the cache
         self._storage._options.cache_servers = 'x:1 y:2'
         self._storage._options.cache_module_name = fakecache.__name__
-        fakecache.data.clear()
 
         db = DB(self._storage)
         try:
             c1 = db.open()
             self.assertEqual(c1._storage._cache_client.servers, ['x:1', 'y:2'])
-            self.assertEqual(len(fakecache.data), 0)
+            fakecache.data.clear()
             r1 = c1.root()
             # the root tid and state should now be cached
             self.assertEqual(len(fakecache.data), 2)
@@ -277,6 +277,18 @@ class RelStorageTests(
         finally:
             db.close()
 
+    def checkLongTransactionDescription(self):
+        # Don't trip over long transaction descriptions
+        db = DB(self._storage)
+        try:
+            c = db.open()
+            r = c.root()
+            r['key'] = 1
+            transaction.get().note('A long description. ' * 1000)
+            transaction.commit()
+        finally:
+            db.close()
+
     def checkAutoReconnect(self):
         # Verify auto-reconnect
         db = DB(self._storage)
@@ -497,6 +509,7 @@ class RelStorageTests(
             while packtime <= now:
                 packtime = time.time()
             self._storage.pack(packtime, referencesf)
+            self._storage.sync()
 
             if expect_object_deleted:
                 # The object should now be gone
@@ -565,7 +578,40 @@ class RelStorageTests(
             self.assertEquals(len(slept), 1)
         finally:
             db.close()
-        
+
+    def checkDoubleCommitter(self):
+        # Verify we can store an object that gets committed twice in
+        # a single transaction.
+        db = DB(self._storage)
+        try:
+            conn = db.open()
+            try:
+                conn.root()['dc'] = DoubleCommitter()
+                transaction.commit()
+                conn2 = db.open()
+                self.assertEquals(conn2.root()['dc'].new_attribute, 1)
+                conn2.close()
+            finally:
+                transaction.abort()
+                conn.close()
+        finally:
+            db.close()
+
+    def checkPackBrokenPickle(self):
+        # Verify the pack stops with the right exception if it encounters
+        # a broken pickle.
+        from cPickle import UnpicklingError
+        self._dostoreNP(self._storage.new_oid(), data='brokenpickle')
+        self.assertRaises(UnpicklingError, self._storage.pack,
+            time.time() + 10000, referencesf)
+
+
+class DoubleCommitter(Persistent):
+    """A crazy persistent class that changes self in __getstate__"""
+    def __getstate__(self):
+        if not hasattr(self, 'new_attribute'):
+            self.new_attribute = 1
+        return Persistent.__getstate__(self)
 
 
 class IteratorDeepCompareUnordered:
